/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_H__
#define __PDB_H__

#ifdef _OLD_HEADER_
# include <iostream.h>
# include <map.h>
# include <vector.h>
# include <mstring.h>
# define STR_NPOS NPOS
#else
# include <iostream>
  using std::ostream;
  using std::cout;
  using std::cerr;
  using std::endl;
# include <map>
  using std::map;
# include <vector>
  using std::vector;
# include <string>
# define STR_NPOS string::npos
  using std::string;
#endif

class PDB;
class pdbSimpleItem;
class pdbItem;
class pdbType;
class pdbFile;
class pdbClass;
class pdbRoutine;
class pdbTemplate;
class pdbMacro;
class pdbNamespace;
class pdbClassField;

template<class tag>
struct PDBTraits {
  typedef int item_t;
};

class PDB {
public:
  struct ltstr {
    bool operator()(const char* s1, const char* s2) const {
      return strcmp(s1, s2) < 0;
    }
  };

  enum attr_t { NOATTR,
       // Common Item Attributes
       ACS, CLASS, LOC, NSPACE, POS,

       // Common Template Item Attributes
       TEMPL, SPECL,

       // Item Specific Attributes
       CBASE, CFRCLASS, CFRFUNC, CFUNC, CMEM, CMCONST, CMISBIT,
       CMKIND, CMMUT, CMTYPE,
       CKIND, MKIND, MTEXT, NALIAS, NMEM,
       RCALL, RCGEN, RCRVO, REXPL, RINLINE, RKIND, RLINK,
       RSIG, RSTAT, RSTORE, RVIRT, SINC, TKIND, TTEXT, YARGT,
       YELEM, YELLIP, YENUM, YEXCEP, YFKIND, YIKIND, YKIND, YMPCLASS,
       YMPTYPE, YNELEM, YQUAL, YPTR, YREF, YRETT, YSIGNED, YTREF
  };

  typedef map<const char*, attr_t, ltstr> attrmap;
  typedef map<attr_t, const char*> namemap;
  typedef vector<pdbType*> typevec;
  typedef vector<pdbFile*> filevec;
  typedef vector<pdbClass*> classvec;
  typedef vector<pdbRoutine*> routinevec;
  typedef vector<pdbTemplate*> templatevec;
  typedef vector<pdbMacro*> macrovec;
  typedef vector<pdbNamespace*> namespacevec;
  typedef vector<pdbSimpleItem*> itemvec;

  typedef map<int, pdbType*> typemap;
  typedef map<int, pdbFile*> filemap;
  typedef map<int, pdbClass*> classmap;
  typedef map<int, pdbRoutine*> routinemap;
  typedef map<int, pdbTemplate*> templatemap;
  typedef map<int, pdbMacro*> macromap;
  typedef map<int, pdbNamespace*> namespacemap;

  PDB(char *fname);
  ~PDB();
  bool write(char *fname);
  void write(ostream& ostr);
  bool merge(char *fname);

  pdbRoutine *callTree()  { return topRoutine; }
  pdbFile    *fileTree()  { return topFile; }
  pdbClass   *classTree() { return topClass; }

  typevec&      getTypeVec()      { return typeVec; }
  filevec&      getFileVec()      { return fileVec; }
  classvec&     getClassVec()     { return classVec; }
  routinevec&   getRoutineVec()   { return routineVec; }
  templatevec&  getTemplateVec()  { return templateVec; }
  macrovec&     getMacroVec()     { return macroVec; }
  namespacevec& getNamespaceVec() { return namespaceVec; }
  itemvec&      getItemVec()      { return itemVec; }

  typemap&      getTypeMap()      { return typeMap; }
  filemap&      getFileMap()      { return fileMap; }
  classmap&     getClassMap()     { return classMap; }
  routinemap&   getRoutineMap()   { return routineMap; }
  templatemap&  getTemplateMap()  { return templateMap; }
  macromap&     getMacroMap()     { return macroMap; }
  namespacemap& getNamespaceMap() { return namespaceMap; }

  operator void *() const { return status ? (void*)&status : 0; }

  static namemap name;
  static attr_t   toAttr(const char *v);

private:
  itemvec      itemVec;
  typevec      typeVec;
  filevec      fileVec;
  classvec     classVec;
  routinevec   routineVec;
  templatevec  templateVec;
  macrovec     macroVec;
  namespacevec namespaceVec;

  static void initAttrs(void);
  static bool is_init;
  static attrmap attr;

  typemap      typeMap;
  filemap      fileMap;
  classmap     classMap;
  routinemap   routineMap;
  templatemap  templateMap;
  macromap     macroMap;
  namespacemap namespaceMap;

  pdbRoutine *topRoutine;
  pdbClass *topClass;
  pdbFile *topFile;
  bool status;

  class typeTag {};
  class fileTag {};
  class classTag {};
  class routineTag {};
  class templateTag {};
  class macroTag {};
  class namespaceTag {};

/*
# ifdef __GNUC__
  friend class PDBTraits<typeTag>;
  friend class PDBTraits<fileTag>;
  friend class PDBTraits<classTag>;
  friend class PDBTraits<routineTag>;
  friend class PDBTraits<templateTag>;
  friend class PDBTraits<macroTag>;
  friend class PDBTraits<namespaceTag>;
# else
  template<class tag> friend class PDBTraits;
# endif
*/
  bool read(char *fname);
  pdbType *getTypeOrClass(const char *value);
  bool getLocation(const char *value, char **ptr,
                   pdbFile*& file, int& line, int& col);

  template<class tag>
  typename PDBTraits<tag>::item_t *findItem(tag t, int id);

  template<class tag>
  typename PDBTraits<tag>::item_t *findItem(tag t, const string& name, int id);

  template<class tag>
  void finalCheck(tag t);

  template<class tag>
  void markDuplicates(tag t, PDB& other);
};

#endif
