/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_CLASS_H__
#define __PDB_CLASS_H__

#ifdef _OLD_HEADER_
# include <vector.h>
#else
# include <vector>
  using std::vector;
#endif
#include "pdbTemplateItem.h"

class pdbBase : public pdbLoc {
public:
  pdbBase() : v(pdbItem::VI_NO), a(pdbItem::AC_NA), bptr(0) {}
  pdbBase(pdbItem::virt_t virt, pdbItem::access_t acs, const pdbClass *base,
          pdbFile *file, int line, int col) : pdbLoc(file, line, col),
          v(virt), a(acs), bptr(base) {}
 
  pdbItem::virt_t virtuality() const { return v; }
  pdbItem::access_t access() const { return a; }
  const pdbClass *base() const { return bptr; }
  bool isVirtual() const { return v == pdbItem::VI_VIRT; }

  void virtuality(pdbItem::virt_t virt) { v = virt; }
  void access(pdbItem::access_t acs) { a = acs; }
  void base(const pdbClass* base) { bptr = base; }

private:
  pdbItem::virt_t   v;
  pdbItem::access_t a;
  const pdbClass   *bptr;
};

class pdbFriendFunc : public pdbLoc {
public:
  pdbFriendFunc() : fuptr(0) {}
  pdbFriendFunc(const pdbRoutine *f, pdbFile *file, int line, int col)
               : pdbLoc(file, line, col), fuptr(f) {}

  const pdbRoutine *friendFunc() const { return fuptr; }
  void friendFunc(const pdbRoutine* f) { fuptr = f; }

private:
  const pdbRoutine *fuptr;
};

class pdbFriendClass : public pdbLoc {
public:
  pdbFriendClass() : cptr(0) {}
  pdbFriendClass(const pdbClass *c, pdbFile *file, int line, int col)
               : pdbLoc(file, line, col), cptr(c) {}

  const pdbClass *friendClass() const { return cptr; }
  void friendClass(const pdbClass* c) { cptr = c; }

private:
  const pdbClass *cptr;
};

class pdbFuncMember : public pdbLoc {
public:
  pdbFuncMember() : fptr(0) {}
  pdbFuncMember(const pdbRoutine *f, pdbFile *file, int line, int col)
               : pdbLoc(file, line, col), fptr(f) {}

  const pdbRoutine *func() const { return fptr; }
  void func(const pdbRoutine* f) { fptr = f; }

private:
  const pdbRoutine *fptr;
};

ostream& operator<<(ostream& ostr, const pdbBase& b);
ostream& operator<<(ostream& ostr, const pdbFriendFunc& f);
ostream& operator<<(ostream& ostr, const pdbFriendClass& c);
ostream& operator<<(ostream& ostr, const pdbFuncMember& f);

class pdbClass : public pdbTemplateItem {
public:
  pdbClass(int id) : pdbTemplateItem(id), abc(false), k(CL_NA) {}
  pdbClass(const string& name, int id) : pdbTemplateItem(name, id),
          abc(false), k(CL_NA) {}
  virtual ~pdbClass();

  virtual const char *desc() const { return "class"; }
  virtual ostream& print(ostream& ostr) const;
  virtual void adjustPtrs(PDB* p);
  virtual void process(PDB* p);
  virtual dupl_t findDuplicate(pdbSimpleItem* r);

  typedef vector<pdbBase *> basevec;
  typedef vector<pdbClass *> classvec;
  typedef vector<pdbFuncMember *> methodvec;
  typedef vector<pdbClassField *> fieldvec;
  typedef vector<pdbFriendClass *> friendclassvec;
  typedef vector<pdbFriendFunc *> friendfuncvec;

  const basevec& baseClasses() const { return b; }
  classvec& derivedClasses() { return s; }
  const classvec& derivedClasses() const { return s; }
  const methodvec& funcMembers() const { return f; }
  const fieldvec& dataMembers() const { return d; }
  bool isABC() const { return abc; }
  class_t kind() const { return k; }
  const friendclassvec& friendClasses() const { return fcl; }
  const friendfuncvec& friendRoutines() const { return fro; }

  void addBaseClass(pdbItem::virt_t virt, pdbItem::access_t acs,
                    const pdbClass *base, pdbFile *file, int line, int col) {
    b.push_back(new pdbBase(virt, acs, base, file, line, col));
  }
  void addDerivedClass(pdbClass *drvd) { s.push_back(drvd); }
  void addFuncMember(pdbRoutine *func, pdbFile *file, int line, int col) {
    f.push_back(new pdbFuncMember(func, file, line, col));
  }
  void addDataMember(pdbClassField *field) { d.push_back(field); }
  void kind(class_t kd) { k = kd; }
  void addFriendClass(pdbClass *f, pdbFile *file, int line, int col) {
    fcl.push_back(new pdbFriendClass(f, file, line, col));
  }
  void addFriendRoutine(pdbRoutine *f, pdbFile *file, int line, int col) { 
    fro.push_back(new pdbFriendFunc(f, file, line, col));
  }

private:
  basevec            b;
  classvec           s;
  methodvec          f;
  fieldvec           d;
  bool               abc;
  class_t            k;
  friendclassvec     fcl;
  friendfuncvec      fro;
};

#endif
