/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_ROUTINE_H__
#define __PDB_ROUTINE_H__

#include "pdbTemplateItem.h"

class pdbCallee : public pdbLoc {
public:
  pdbCallee () : v(pdbItem::VI_NO), rptr(0)  {}
  pdbCallee (const pdbRoutine *call, pdbItem::virt_t virt,
             pdbFile *file, int line, int col)
            : pdbLoc(file, line, col), v(virt), rptr(call) {}
  
  pdbItem::virt_t virtuality() const { return v; }
  const pdbRoutine *call() const { return rptr; }
  bool isVirtual() const { return v == pdbItem::VI_VIRT; }

  void virtuality(pdbItem::virt_t virt) { v = virt; }
  void call(const pdbRoutine* call) { rptr = call; }

private:
  pdbItem::virt_t v;
  const pdbRoutine *rptr;
};

ostream& operator<<(ostream& ostr, const pdbCallee& c);

class pdbRoutine : public pdbTemplateItem {
public:
  pdbRoutine(int id) : pdbTemplateItem(id),
            sig(0), store(ST_NA), k(FU_NA), v(VI_NO), crvo(false),
            i(false), isCGen(false), isEx(false), isStat(false), n(0),
	    l(LK_NA) {}
  pdbRoutine(const string& name, int id) : pdbTemplateItem(name, id),
            sig(0), store(ST_NA), k(FU_NA), v(VI_NO), crvo(false),
            i(false), isCGen(false), isEx(false), isStat(false), n(0),
	    l(LK_NA) {}
  virtual ~pdbRoutine();

  virtual ostream& print(ostream& ostr) const;
  virtual const char *desc() const { return "routine"; }
  virtual void adjustPtrs(PDB* p);
  virtual dupl_t findDuplicate(pdbSimpleItem* r);

  typedef vector<pdbCallee *> callvec;

  const pdbType *signature() const { return sig; }
  store_t storageClass() const { return store; }
  func_t specialKind() const { return k; }
  virt_t virtuality() const { return v; }
  bool covariantReturnVirtualOverride() const { return crvo; }
  bool isInline() const { return i; }
  callvec& callees() { return c; }
  const callvec& callees() const { return c; }
  int numCalled() const { return n; }
  bool isCompilerGenerated() const { return isCGen; }
  bool isExplicitCtor() const { return isEx; }
  bool isStatic() const { return isStat; }
  link_t linkage() const { return l; }
  bool isVirtual() const {
    return v == pdbItem::VI_VIRT || v == pdbItem::VI_PURE;
  }

  void signature(const pdbType *sigtype) { sig = sigtype; }
  void storageClass(store_t sclass) { store = sclass; }
  void specialKind(func_t kind) { k = kind; }
  void virtuality(virt_t virt) { v = virt; }
  void covariantReturnVirtualOverride(bool c) { crvo = c; }
  void isInline(bool inl) { i = inl; }
  void addCallee(const pdbRoutine *callee, pdbItem::virt_t virt,
                 pdbFile *file, int line, int col) {
    c.push_back(new pdbCallee(callee, virt, file, line, col));
    callee->incrCalled();
  }
  void isCompilerGenerated(bool c) { isCGen = c; }
  void isExplicitCtor(bool e) { isEx = e; }
  void isStatic(bool s) { isStat = s; }
  void linkage(link_t link) { l = link; }

  virtual void process(PDB *p);

protected:
  void incrCalled() const;

  const pdbType     *sig;
  store_t            store;
  func_t             k;
  virt_t             v;
  bool               crvo;
  bool               i;
  bool               isCGen;
  bool               isEx;
  bool               isStat;
  callvec            c;
  int                n;
  link_t             l;
};

#endif
