/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifndef __PDB_TYPE_H__
#define __PDB_TYPE_H__

#include "pdbItem.h"

class pdbEnum {
public:
  pdbEnum(const char* id, int val) : i(id), v(val) {}
  const string& id() const { return i; }
  int val() const { return v; }
private:
  string i;
  int    v;
};

class pdbArgument {
public:
  pdbArgument(const pdbType *type, bool hasDefault = false)
	     : t(type), d(hasDefault) {}
  const pdbType* type() const { return t; }
  bool hasDefault() const { return d; }

  void type(const pdbType *typ) { t = typ; }
private:
  const pdbType *t;
  bool           d;
};

class pdbType : public pdbItem {
public:
  pdbType(int id) : pdbItem(id), e(0), f(FL_NA), i(I_NA), t(TY_NA),
                    p(0), snd(false), b(0), c(0), r(0), ell(false), mt(0),
                    mc(0), cst(false), ne(0) {}
  pdbType(const string& name, int id) : pdbItem(name, id), e(0), f(FL_NA),
                    i(I_NA), t(TY_NA), p(0), snd(false), b(0), c(0),
		    r(0), ell(false), mt(0), mc(0), cst(false), ne(0) {}

  virtual const char *desc() const { return "type"; }
  virtual const char *attrPrefix() const { return "y"; }
  virtual ostream& print(ostream& ostr) const;
  virtual void adjustPtrs(PDB *p);
  virtual dupl_t findDuplicate(pdbSimpleItem* r);
  virtual void process(PDB *p);

  typedef vector<qual_t> qualvec;
  typedef vector<pdbArgument> argvec;
  typedef vector<const pdbClass *> classvec;
  typedef vector<pdbEnum> enumvec;

  static const pdbClass* NULLEXCEP;

  const pdbType* elementType() const { return e; }
  float_t floatType() const { return f; }
  int_t integerType() const { return i; }
  type_t kind() const { return t; }
  const qualvec& qualifiers() const  { return q; }
  const pdbType* pointedToType() const { return (t==TY_PTR)?p:0; }
  const pdbType* referencedType() const { return (t==TY_REF)?p:0; }
  bool isSigned() const { return snd; }
  const pdbType* baseType() const { return b; }
  const pdbClass* isClass() const { return c; }
  const pdbType* returnType() const { return r; }
  const argvec& argumentTypes() const { return a; }
  bool hasEllipsis() const { return ell; }
  const pdbType* memberPointerType() const { return mt; }
  const pdbClass* memberPointerClass() const { return mc; }
  bool isConst() const { return cst; }
  int numElements() const { return ne; }
  const classvec& exceptionSpec() const { return es; }
  const enumvec& enumeration() const { return en; }

  void elementType(const pdbType* elem) { e = elem; }
  void floatType(float_t ftype) { f = ftype; }
  void integerType(int_t itype) { i = itype; }
  void kind(type_t type) { t = type; }
  void addQualifier(qual_t qual) { q.push_back(qual); }
  void pointedToType(const pdbType* ptr) { p = ptr; }
  void referencedType(const pdbType* ptr) { p = ptr; }
  void isSigned(bool s) { snd = s; }
  void baseType(const pdbType* base) { b = base; }
  void isClass(const pdbClass* cl) { c = cl; }
  void returnType(const pdbType* type) { r = type; }
  void addArgumentType(const pdbType* type, bool hasDefault = false) {
    a.push_back(pdbArgument(type, hasDefault));
  }
  void hasEllipsis(bool e) { ell = e; }
  void memberPointerType(const pdbType* mptype) { mt = mptype; }
  void memberPointerClass(const pdbClass* mpclass) { mc = mpclass; }
  void isConst(bool c) { cst = c; }
  void numElements(int n) { ne = n; }
  void addExceptionSpec(const pdbClass* c) { es.push_back(c); }
  void addEnumeration(const pdbEnum& e) { en.push_back(e); }

private:
  const pdbType  *e;
  float_t         f;
  int_t           i;
  type_t          t;
  qualvec         q;
  const pdbType  *p;
  bool            snd;
  const pdbType  *b;
  const pdbClass *c;
  const pdbType  *r;
  argvec          a;
  bool            ell;
  const pdbType  *mt;
  const pdbClass *mc;
  bool            cst;
  int             ne;
  classvec        es;
  enumvec         en;
};

#endif
