/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#include "pdbItem.h"
#include "pdbClass.h"
#include "pdbRoutine.h"
#include "pdbNamespace.h"

extern void ERROR(const char *msg, const char *val);
extern void ERROR(const char *msg, const char *item, int val);

void pdbItem::process(PDB*) {
  string fn = name();
  const pdbItem *iptr = this, *jptr;
  
  do {
    if ( iptr->name().rfind("::") != STR_NPOS ) break; // ??? parser bug
    if ( (jptr = iptr->parentClass()) )
      fn = jptr->name() + "::" + fn;
    else if ( (jptr = iptr->parentNSpace()) )
      fn = jptr->name() + "::" + fn;
    iptr = jptr;
  } while (iptr);
      
  fullName(fn);
}

ostream& pdbItem::print(ostream& ostr) const {
  const char *X = attrPrefix();
  ostr << desc()[0] << desc()[1] << "#" << id() << " ";
  if ( ! (name() == "-") ) ostr << name();
  ostr << "\n";
  if ( location().file() ) ostr << X << "loc " << location() << "\n";
  if ( const pdbClass* cptr = parentClass() ) {
    ostr << X << "class cl#" << cptr->id() << "\n";
    ostr << X << "acs " << toName(access()) << "\n";
  }
  if ( const pdbNamespace* nptr = parentNSpace() ) {
    ostr << X << "nspace na#" << nptr->id() << "\n";
  }
  return ostr;
}

void pdbItem::adjustPtrs(PDB* p) {
  const pdbFile* f;
  if ( (f=l.file()) && (f->newId() > UNIQUE) )
    l.file(p->getFileMap()[f->newId()]);
  if ( cptr && (cptr->newId() > UNIQUE) )
    cptr = p->getClassMap()[cptr->newId()];
  if ( nptr && (nptr->newId() > UNIQUE) )
    nptr = p->getNamespaceMap()[nptr->newId()];
}

const char *pdbItem::toName(access_t v) {
  switch (v) {
  case AC_PUB : return "pub";
  case AC_PRIV: return "priv";
  case AC_PROT: return "prot";
  case AC_NA  :
  default     : return "NA";
  }
}

const char *pdbItem::toName(store_t v) {
  switch (v) {
  case ST_ASM : return "asm";
  case ST_AUTO: return "auto";
  case ST_EXT : return "ext";
  case ST_STAT: return "stat";
  case ST_NA  :
  default     : return "NA";
  }
}

const char *pdbItem::toName(func_t v) {
  switch (v) {
  case FU_CONV: return "conv";
  case FU_CTOR: return "ctor";
  case FU_DTOR: return "dtor";
  case FU_OP  : return "op";
  case FU_NA  :
  default     : return "NA";
  }
}

const char *pdbItem::toName(virt_t v) {
  switch (v) {
  case VI_PURE: return "pure";
  case VI_VIRT: return "virt";
  case VI_NO  :
  default     : return "no";
  }
}

const char *pdbItem::toName(macro_t v) {
  switch (v) {
  case MA_DEF  : return "def";
  case MA_UNDEF: return "undef";
  case MA_NA   :
  default      : return "NA";
  }
}

const char *pdbItem::toName(templ_t v) {
  switch (v) {
  case TE_CLASS   : return "class";
  case TE_FUNC    : return "func";
  case TE_MEMCLASS: return "memclass";
  case TE_MEMFUNC : return "memfunc";
  case TE_STATMEM : return "statmem";
  case TE_NA      :
  default         : return "NA";
  }
}

const char *pdbItem::toName(float_t v) {
  switch (v) {
  case FL_FLOAT  : return "float";
  case FL_DBL    : return "dbl";
  case FL_LONGDBL: return "longdbl";
  case FL_NA     :
  default        : return "NA";
  }
}

const char *pdbItem::toName(int_t v) {
  switch (v) {
  case I_CHAR     : return "char";
  case I_SCHAR    : return "schar";
  case I_UCHAR    : return "uchar";
  case I_SHORT    : return "short";
  case I_USHORT   : return "ushort";
  case I_INT      : return "int";
  case I_UINT     : return "uint";
  case I_LONG     : return "long";
  case I_ULONG    : return "ulong";
  case I_LONGLONG : return "longlong";
  case I_ULONGLONG: return "ulonglong";
  case I_NA       :
  default         : return "NA";
  }
}

const char *pdbItem::toName(type_t v) {
  switch (v) {
  case TY_BOOL  : return "bool";
  case TY_ENUM  : return "enum";
  case TY_ERR   : return "err";
  case TY_FUNC  : return "func";
  case TY_VOID  : return "void";
  case TY_INT   : return "int";
  case TY_FLOAT : return "float";
  case TY_PTR   : return "ptr";
  case TY_REF   : return "ref";
  case TY_ARRAY : return "array";
  case TY_TREF  : return "tref";
  case TY_PTRMEM: return "ptrmem";
  case TY_TPARAM: return "tparam";
  case TY_WCHAR : return "wchar";
  case TY_CLASS : return "class";
  case TY_NA    :
  default       : return "NA";
  }
}

const char *pdbItem::toName(qual_t v) {
  switch (v) {
  case QL_CONST   : return "const";
  case QL_VOLATILE: return "volatile";
  case QL_RESTRICT: return "restrict";
  default         : return "NA";
  }
}

const char *pdbItem::toName(class_t v) {
  switch (v) {
  case CL_CLASS : return "class";
  case CL_STRUCT: return "struct";
  case CL_UNION : return "union";
  case CL_NA    : 
  default       : return "NA";
  }
}

const char *pdbItem::toName(link_t v) {
  switch (v) {
  case LK_INTERNAL: return "internal";
  case LK_CXX     : return "C++";
  case LK_C       : return "C";
  case LK_NA      :
  default         : return "no";
  }
}

const char *pdbItem::toName(mem_t v) {
  switch (v) {
  case M_VAR    : return "var";
  case M_STATVAR: return "statvar";
  case M_TYPE   : return "type";
  case M_NA     :
  default       : return "NA";
  }
}

pdbItem::access_t pdbItem::toAccess(const char *v) {
  if ( strcmp("pub",  v) == 0 ) return AC_PUB;
  if ( strcmp("priv", v) == 0 ) return AC_PRIV;
  if ( strcmp("prot", v) == 0 ) return AC_PROT;
  if ( strcmp("NA",   v) == 0 ) return AC_NA;
  ERROR("Unknown access mode ", v);
  return AC_NA;
}

pdbItem::store_t pdbItem::toStore(const char *v) {
  if ( strcmp("ext",  v) == 0 ) return ST_EXT;
  if ( strcmp("stat", v) == 0 ) return ST_STAT;
  if ( strcmp("auto", v) == 0 ) return ST_AUTO;
  if ( strcmp("asm",  v) == 0 ) return ST_ASM;
  if ( strcmp("NA",   v) == 0 ) return ST_NA;
  ERROR("Unknown storage mode ", v);
  return ST_NA;
}

pdbItem::func_t pdbItem::toFunc(const char *v) {
  if ( strcmp("ctor", v) == 0 ) return FU_CTOR;
  if ( strcmp("dtor", v) == 0 ) return FU_DTOR;
  if ( strcmp("op",   v) == 0 ) return FU_OP;
  if ( strcmp("conv", v) == 0 ) return FU_CONV;
  if ( strcmp("NA",   v) == 0 ) return FU_NA;
  ERROR("Unknown routine kind ", v);
  return FU_NA;
}

pdbItem::virt_t pdbItem::toVirt(const char *v) {
  if ( strcmp("no",   v) == 0 ) return VI_NO;
  if ( strcmp("virt", v) == 0 ) return VI_VIRT;
  if ( strcmp("pure", v) == 0 ) return VI_PURE;
  ERROR("Unknown virtuality mode ", v);
  return VI_NO;
}

pdbItem::macro_t pdbItem::toMacro(const char *v) {
  if ( strcmp("def",   v) == 0 ) return MA_DEF;
  if ( strcmp("undef", v) == 0 ) return MA_UNDEF;
  if ( strcmp("NA",    v) == 0 ) return MA_NA;
  ERROR("Unknown macro kind ", v);
  return MA_NA;
}

pdbItem::templ_t pdbItem::toTempl(const char *v) {
  if ( strcmp("class",    v) == 0 ) return TE_CLASS;
  if ( strcmp("func",     v) == 0 ) return TE_FUNC;
  if ( strcmp("memclass", v) == 0 ) return TE_MEMCLASS;
  if ( strcmp("memfunc",  v) == 0 ) return TE_MEMFUNC;
  if ( strcmp("statmem",  v) == 0 ) return TE_STATMEM;
  if ( strcmp("none",     v) == 0 ) return TE_NA;
  ERROR("Unknown template kind ", v);
  return TE_NA;
}

pdbItem::float_t pdbItem::toFloat(const char* v) {
  if ( strcmp("float",   v) == 0 ) return FL_FLOAT;
  if ( strcmp("dbl",     v) == 0 ) return FL_DBL;
  if ( strcmp("longdbl", v) == 0 ) return FL_LONGDBL;
  ERROR("Unknown float kind ", v);
  return FL_NA;
}

pdbItem::int_t pdbItem::toInt(const char* v) {
  if ( strcmp("char",      v) == 0 ) return I_CHAR;
  if ( strcmp("schar",     v) == 0 ) return I_SCHAR;
  if ( strcmp("uchar",     v) == 0 ) return I_UCHAR;
  if ( strcmp("short",     v) == 0 ) return I_SHORT;
  if ( strcmp("ushort",    v) == 0 ) return I_USHORT;
  if ( strcmp("int",       v) == 0 ) return I_INT;
  if ( strcmp("uint",      v) == 0 ) return I_UINT;
  if ( strcmp("long",      v) == 0 ) return I_LONG;
  if ( strcmp("ulong",     v) == 0 ) return I_ULONG;
  if ( strcmp("longlong",  v) == 0 ) return I_LONGLONG;
  if ( strcmp("ulonglong", v) == 0 ) return I_ULONGLONG;
  ERROR("Unknown integer kind ", v);
  return I_NA;
}

pdbItem::type_t pdbItem::toType(const char* v) {
  if ( strcmp("bool",   v) == 0 ) return TY_BOOL;
  if ( strcmp("enum",   v) == 0 ) return TY_ENUM;
  if ( strcmp("err",    v) == 0 ) return TY_ERR;
  if ( strcmp("func",   v) == 0 ) return TY_FUNC;
  if ( strcmp("void",   v) == 0 ) return TY_VOID;
  if ( strcmp("int",    v) == 0 ) return TY_INT;
  if ( strcmp("float",  v) == 0 ) return TY_FLOAT;
  if ( strcmp("ptr",    v) == 0 ) return TY_PTR;
  if ( strcmp("ref",    v) == 0 ) return TY_REF;
  if ( strcmp("array",  v) == 0 ) return TY_ARRAY;
  if ( strcmp("tref",   v) == 0 ) return TY_TREF;
  if ( strcmp("ptrmem", v) == 0 ) return TY_PTRMEM;
  if ( strcmp("tparam", v) == 0 ) return TY_TPARAM;
  if ( strcmp("wchar",  v) == 0 ) return TY_WCHAR;
  ERROR("Unknown type kind ", v);
  return TY_NA;
}

pdbItem::qual_t pdbItem::toQual(const char* v) {
  if ( strcmp("const",    v) == 0 ) return QL_CONST;
  if ( strcmp("volatile", v) == 0 ) return QL_VOLATILE;
  if ( strcmp("restrict", v) == 0 ) return QL_RESTRICT;
  ERROR("Unknown qualifier ", v);
  return QL_NA;
}

pdbItem::class_t pdbItem::toClass(const char* v) {
  if ( strcmp ("class",  v) == 0 ) return CL_CLASS;
  if ( strcmp ("struct", v) == 0 ) return CL_STRUCT;
  if ( strcmp ("union",  v) == 0 ) return CL_UNION;
  ERROR("Unknown class kind ", v);
  return CL_NA;
}

pdbItem::link_t pdbItem::toLink(const char* v) {
  if ( strcmp ("internal", v) == 0 ) return LK_INTERNAL;
  if ( strcmp ("C++",      v) == 0 ) return LK_CXX;
  if ( strcmp ("C",        v) == 0 ) return LK_C;
  if ( strcmp ("no",       v) == 0 ) return LK_NA;
  ERROR("Unknown linkage kind ", v);
  return LK_NA;
}

pdbItem::mem_t pdbItem::toMem(const char* v) {
  if ( strcmp ("var",     v) == 0 ) return M_VAR;
  if ( strcmp ("statvar", v) == 0 ) return M_STATVAR;
  if ( strcmp ("type",    v) == 0 ) return M_TYPE;
  ERROR("Unknown class member type ", v);
  return M_NA;
}

