/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#include "pdb.h"
#include "pdbRoutine.h"
#include "pdbClass.h"

PDB::PDB(char *fname) {
  if (!is_init) initAttrs();
  is_init = true;
  topRoutine = new pdbRoutine("$TOP$", pdbSimpleItem::UNIQUE);
  topClass = new pdbClass("$TOP$", pdbSimpleItem::UNIQUE);
  topFile = new pdbFile("$TOP$", pdbSimpleItem::UNIQUE);
  status = read(fname);
}

PDB::~PDB() {
  for (itemvec::iterator it = itemVec.begin(); it!=itemVec.end(); ++it) {
    if ( (*it)->newId() == pdbSimpleItem::NOTDEL ) {
      // do not delete: item used in other PDB; but reset
      (*it)->newId(pdbSimpleItem::UNIQUE);
    } else {
      delete *it;
    }
  }
  delete topRoutine;
  delete topClass;
  delete topFile;
}

PDB::attrmap PDB::attr;
PDB::namemap PDB::name;
bool PDB::is_init = false;

void PDB::initAttrs(void) {

  // Common Attributes 
  attr["cacs"]     = ACS;       name[ACS]       = "acs";
  attr["cmacs"]    = ACS;
  attr["racs"]     = ACS;
  attr["tacs"]     = ACS;
  attr["yacs"]     = ACS;
  attr["cclass"]   = CLASS;     name[CLASS]     = "class";
  attr["rclass"]   = CLASS;
  attr["tclass"]   = CLASS;
  attr["yclass"]   = CLASS;
  attr["cmloc"]    = LOC;       name[LOC]       = "loc";
  attr["cloc"]     = LOC;
  attr["mloc"]     = LOC;
  attr["nloc"]     = LOC;
  attr["rloc"]     = LOC;
  attr["tloc"]     = LOC;
  attr["yloc"]     = LOC;
  attr["cnspace"]  = NSPACE;    name[NSPACE]    = "nspace";
  attr["nnspace"]  = NSPACE;
  attr["rnspace"]  = NSPACE;
  attr["tnspace"]  = NSPACE;
  attr["ynspace"]  = NSPACE;
  attr["rpos"]     = POS;       name[POS]       = "pos";
  attr["cpos"]     = POS;
  attr["tpos"]     = POS;
  attr["npos"]     = POS;

  // Common Template Item Attributes
  attr["cmspecl"]  = SPECL;     name[SPECL]     = "specl";
  attr["cspecl"]   = SPECL;
  attr["rspecl"]   = SPECL;
  attr["cmtempl"]  = TEMPL;     name[TEMPL]     = "templ";
  attr["ctempl"]   = TEMPL;
  attr["rtempl"]   = TEMPL;

  // Item Specific Attributes
  attr["cbase"]    = CBASE;     name[CBASE]     = "cbase";
  attr["cfrclass"] = CFRCLASS;  name[CFRCLASS]  = "cfrclass";
  attr["cfrfunc"]  = CFRFUNC;   name[CFRFUNC]   = "cfrfunc";
  attr["cfunc"]    = CFUNC;     name[CFUNC]     = "cfunc";
  attr["ckind"]    = CKIND;     name[CKIND]     = "ckind";
  attr["cmem"]     = CMEM;      name[CMEM]      = "cmem";
  attr["cmconst"]  = CMCONST;   name[CMCONST]   = "cmconst";
  attr["cmisbit"]  = CMISBIT;   name[CMISBIT]   = "cmisbit";
  attr["cmkind"]   = CMKIND;    name[CMKIND]    = "cmkind";
  attr["cmmut"]    = CMMUT;     name[CMMUT]     = "cmmut";
  attr["cmtype"]   = CMTYPE;    name[CMTYPE]    = "cmtype";
  attr["mkind"]    = MKIND;     name[MKIND]     = "mkind";
  attr["mtext"]    = MTEXT;     name[MTEXT]     = "mtext";
  attr["nalias"]   = NALIAS;    name[NALIAS]    = "nalias";
  attr["nmem"]     = NMEM;      name[NMEM]      = "nmem";
  attr["rcall"]    = RCALL;     name[RCALL]     = "rcall";
  attr["rcgen"]    = RCGEN;     name[RCGEN]     = "rcgen";
  attr["rcrvo"]    = RCRVO;     name[RCRVO]     = "rcrvo";
  attr["rexpl"]    = REXPL;     name[REXPL]     = "rexpl";
  attr["rinline"]  = RINLINE;   name[RINLINE]   = "rinline";
  attr["rkind"]    = RKIND;     name[RKIND]     = "rkind";
  attr["rlink"]    = RLINK;     name[RLINK]     = "rlink";
  attr["rsig"]     = RSIG;      name[RSIG]      = "rsig";
  attr["rstatic"]  = RSTAT;     name[RSTAT]     = "rstatic";
  attr["rstore"]   = RSTORE;    name[RSTORE]    = "rstore";
  attr["rvirt"]    = RVIRT;     name[RVIRT]     = "rvirt";
  attr["sinc"]     = SINC;      name[SINC]      = "sinc";
  attr["tkind"]    = TKIND;     name[TKIND]     = "tkind";
  attr["ttext"]    = TTEXT;     name[TTEXT]     = "ttext";
  attr["yargt"]    = YARGT;     name[YARGT]     = "yargt";
  attr["yelem"]    = YELEM;     name[YELEM]     = "yelem";
  attr["yellip"]   = YELLIP;    name[YELLIP]    = "yellip";
  attr["yenum"]    = YENUM;     name[YENUM]     = "yenum";
  attr["yexcep"]   = YEXCEP;    name[YEXCEP]    = "yexcep";
  attr["yfkind"]   = YFKIND;    name[YFKIND]    = "yfkind";
  attr["yikind"]   = YIKIND;    name[YIKIND]    = "yikind";
  attr["ykind"]    = YKIND;     name[YKIND]     = "ykind";
  attr["ympclass"] = YMPCLASS;  name[YMPCLASS]  = "ympclass";
  attr["ymptype"]  = YMPTYPE;   name[YMPTYPE]   = "ymptype";
  attr["ynelem"]   = YNELEM;    name[YNELEM]    = "ynelem";
  attr["yqual"]    = YQUAL;     name[YQUAL]     = "yqual";
  attr["yptr"]     = YPTR;      name[YPTR]      = "yptr";
  attr["yref"]     = YREF;      name[YREF]      = "yref";
  attr["yrett"]    = YRETT;     name[YRETT]     = "yrett";
  attr["ysigned"]  = YSIGNED;   name[YSIGNED]   = "ysigned";
  attr["ytref"]    = YTREF;     name[YTREF]     = "ytref";
}

