/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#include "pdbNamespace.h"
#include "pdbClass.h"
#include "pdbType.h"
#include "pdbRoutine.h"
#include "pdbTemplate.h"

ostream& pdbNamespace::print(ostream& ostr) const {
  pdbItem::print(ostr);
  if ( isAlias() ) ostr << "nalias na#" << ns->id() << "\n";
  for (memvec::const_iterator it = members().begin();
      it!=members().end(); ++it) {
    ostr << "nmem " << (*it)->desc()[0] << (*it)->desc()[1] << "#"
         << (*it)->id() << "\n";
  }
  pdbFatItem::print(ostr);
  return ostr << endl;
}

void pdbNamespace::adjustPtrs(PDB* p) {
  pdbFatItem::adjustPtrs(p);
  if ( ns && ns->newId() > UNIQUE ) ns = p->getNamespaceMap()[ns->newId()];
  for( memvec::iterator it = m.begin(); it!=m.end(); ++it) {
    if ( (*it)->newId() > UNIQUE ) {
      if ( dynamic_cast<pdbRoutine*>(*it) )
        *it = p->getRoutineMap()[(*it)->newId()];
      else if ( dynamic_cast<pdbClass*>(*it) )
        *it = p->getClassMap()[(*it)->newId()];
      else if ( dynamic_cast<pdbType*>(*it) )
        *it = p->getTypeMap()[(*it)->newId()];
      else if ( dynamic_cast<pdbTemplate*>(*it) )
        *it = p->getTemplateMap()[(*it)->newId()];
      else if ( dynamic_cast<pdbNamespace*>(*it) )
        *it = p->getNamespaceMap()[(*it)->newId()];
    }
  }
}

pdbSimpleItem::dupl_t pdbNamespace::findDuplicate(pdbSimpleItem* r) {
  pdbNamespace* rhs = dynamic_cast<pdbNamespace*> (r);
  if ( fullName() == rhs->fullName() ) {
    if ( name() == "-" )         // unnamed namespace
      return NODUPL;
    else
      return NEWDUPL;
  } else {
      return NODUPL;
  }
}
