/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#include <string.h>
#include "pdbTemplate.h"

ostream& pdbTemplate::print(ostream& ostr) const {
  pdbItem::print(ostr);
  ostr << "tkind " << toName(kind()) << "\n";
  if ( ! (text() == "") ) ostr << "ttext " << text() << "\n";
  pdbFatItem::print(ostr);
  return ostr << endl;
}

void pdbTemplate::process(PDB *p) {
  pdbItem::process(p);

  // add template param /*???*/
  const string& strtext = text();
  if ( ! (strtext == "") ) {
    int start = 9;
    int end = start+1;
    int cnt = 1;
    while ( strtext[end] ) {
      if ( strtext[end] == '<' )
        ++cnt;
      else if ( strtext[end] == '>' ) {
        --cnt;
        if ( cnt == 0 ) break;
      }
      ++end;
    }
    string targ = text().substr(start, end-start+1);

    // remove possible line breaks
    bool blank = false;
    unsigned i, j = 0;
    for (i=0; i<targ.size(); ++i) {
      if ( targ[i] == '\\' ) {
        targ[j++] = ' ';
        i += 3;
        blank = true;
      } else if ( targ[i] == ' ' ) {
        if (!blank) targ[j++] = ' ';
        blank = true;
      } else {
        targ[j++] = targ[i];
        blank = false;
      }
    }
    if ( j < i) targ.resize(j);
    fullName(fullName() + targ);
  }
}

pdbSimpleItem::dupl_t pdbTemplate::findDuplicate(pdbSimpleItem* r) {
  pdbTemplate* rhs = dynamic_cast<pdbTemplate*> (r);
  // ??? for function templates, should actually compare "signatures"
  // ??? but this information is not available yet
  if ( fullName() == rhs->fullName() ) {
    pdbLoc ll = location();
    pdbLoc rl = rhs->location();
    if ( (ll.file() && rl.file()) && (ll.file()->name() == rl.file()->name()) &&
         (ll.line() == rl.line()) && (ll.col() == rl.col()) ) {
      return NEWDUPL;
    } else
      return NODUPL;
  } else {
    return NODUPL;
  }
}
