/*************************************************************************/
/* DUCTAPE Version 1.0                                                   */
/* Copyright (C) 1998                                                    */
/* Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik */
/* Los Alamos National Laboratory, Advanced Computing Laboratory         */
/*************************************************************************/

#ifdef _OLD_HEADER_
# include <fstream.h>
#else
# include <fstream>
  using std::ofstream;
#endif
#include <unistd.h>
#include <stdlib.h>
#include "pdbAll.h"

bool Aflag = false;
bool Cflag = false;
bool Mflag = false;
bool Nflag = false;
bool Rflag = false;
bool Sflag = false;
bool Tflag = false;
bool Yflag = false;

static char toUpper(const char ch) { return (ch - 'a' + 'A'); }

static const char *toName(pdbItem::access_t v) {
  switch (v) {
  case pdbItem::AC_PUB : return "public";
  case pdbItem::AC_PRIV: return "private";
  case pdbItem::AC_PROT: return "protected";
  case pdbItem::AC_NA  :
  default              : return "NA";
  }
}

static const char *toName(pdbItem::store_t v) {
  switch (v) {
  case pdbItem::ST_ASM : return "assembler";
  case pdbItem::ST_AUTO: return "automatic";
  case pdbItem::ST_EXT : return "extern";
  case pdbItem::ST_STAT: return "static";
  case pdbItem::ST_NA  :
  default              : return "NA";
  }
}

static const char *toName(pdbItem::func_t v) {
  switch (v) {
  case pdbItem::FU_CONV: return "conversionOperator";
  case pdbItem::FU_CTOR: return "constructor";
  case pdbItem::FU_DTOR: return "destructor";
  case pdbItem::FU_OP  : return "operator";
  case pdbItem::FU_NA  :
  default              : return "NA";
  }
}

static const char *toName(pdbItem::virt_t v) {
  switch (v) {
  case pdbItem::VI_PURE: return "pureVirtual";
  case pdbItem::VI_VIRT: return "virtual";
  case pdbItem::VI_NO  :
  default              : return "no";
  }
}

static const char *toName(pdbItem::macro_t v) {
  switch (v) {
  case pdbItem::MA_DEF  : return "define";
  case pdbItem::MA_UNDEF: return "undef";
  case pdbItem::MA_NA   :
  default               : return "NA";
  }
}

static const char *toName(pdbItem::templ_t v) {
  switch (v) {
  case pdbItem::TE_CLASS   : return "class";
  case pdbItem::TE_FUNC    : return "function";
  case pdbItem::TE_MEMCLASS: return "memberClass";
  case pdbItem::TE_MEMFUNC : return "memberFunction";
  case pdbItem::TE_STATMEM : return "staticDataMember";
  case pdbItem::TE_NA      :
  default                  : return "NA";
  }
}

static const char *toName(pdbItem::float_t v) {
  switch (v) {
  case pdbItem::FL_FLOAT  : return "float";
  case pdbItem::FL_DBL    : return "double";
  case pdbItem::FL_LONGDBL: return "longDouble";
  case pdbItem::FL_NA     :
  default                 : return "NA";
  }
}

static const char *toName(pdbItem::int_t v) {
  switch (v) {
  case pdbItem::I_CHAR     : return "character";
  case pdbItem::I_SCHAR    : return "signedCharacter";
  case pdbItem::I_UCHAR    : return "unsignedCharacter";
  case pdbItem::I_SHORT    : return "short";
  case pdbItem::I_USHORT   : return "unsignedShort";
  case pdbItem::I_INT      : return "integer";
  case pdbItem::I_UINT     : return "unsignedInteger";
  case pdbItem::I_LONG     : return "long";
  case pdbItem::I_ULONG    : return "uunsignedLong";
  case pdbItem::I_LONGLONG : return "longLong";
  case pdbItem::I_ULONGLONG: return "unsignedLongLong";
  case pdbItem::I_NA       :
  default                  : return "NA";
  }
}

static const char *toName(pdbItem::type_t v) {
  switch (v) {
  case pdbItem::TY_BOOL  : return "boolean";
  case pdbItem::TY_ENUM  : return "enumeration";
  case pdbItem::TY_ERR   : return "error";
  case pdbItem::TY_FUNC  : return "functionSignature";
  case pdbItem::TY_VOID  : return "void";
  case pdbItem::TY_INT   : return "integer";
  case pdbItem::TY_FLOAT : return "float";
  case pdbItem::TY_PTR   : return "pointer";
  case pdbItem::TY_REF   : return "reference";
  case pdbItem::TY_ARRAY : return "array";
  case pdbItem::TY_TREF  : return "typeReference";
  case pdbItem::TY_PTRMEM: return "pointerToMember";
  case pdbItem::TY_TPARAM: return "templateParameter";
  case pdbItem::TY_WCHAR : return "wideCharacter";
  case pdbItem::TY_CLASS : return "classType";
  case pdbItem::TY_NA    :
  default                : return "NA";
  }
}

static const char *toName(pdbItem::qual_t v) {
  switch (v) {
  case pdbItem::QL_CONST   : return "const";
  case pdbItem::QL_VOLATILE: return "volatile";
  case pdbItem::QL_RESTRICT: return "restrict";
  default                  : return "NA";
  }
}

static const char *toName(pdbItem::class_t v) {
  switch (v) {
  case pdbItem::CL_CLASS : return "class";
  case pdbItem::CL_STRUCT: return "struct";
  case pdbItem::CL_UNION : return "union";
  case pdbItem::CL_NA    : 
  default                : return "NA";
  }
}

static const char *toName(pdbItem::mem_t v) {
  switch (v) {
  case pdbItem::M_VAR    : return "variable";
  case pdbItem::M_STATVAR: return "staticVariable";
  case pdbItem::M_TYPE   : return "type";
  case pdbItem::M_NA     :
  default                : return "NA";
  }
}

static void printType(ostream& ostr, const char *attr, const pdbType *ty,
                      const char *extra = "") {
  const pdbClass *cl;

  if ( (cl = ty->isClass()) != 0 )
    ostr << attr << "CL#" << cl->id() << " " << cl->name() << extra << "\n";
  else
    ostr << attr << "TY#" << ty->id() << " " << ty->name() << extra << "\n";
}

void printPos(ostream& ostr, const pdbLoc& l) {
  if ( l.file() ) {
    ostr << " SO#"
         << l.file()->id() << " " << l.file()->name()
         << " (" << l.line() << "," << l.col() << ")";
  } else {
    ostr << " <UNKNOWN>";
  }
}

void printPositions(ostream& ostr, const pdbFatItem* fi) {
  pdbLoc l;

  ostr << "headPosition:      ";
  printPos(ostr,  fi->headBegin());
  l = fi->headEnd();
  if ( l.file() && fi->headBegin().file() == l.file() )
    ostr << " -> (" << l.line() << "," << l.col() << ")";
  else
    printPos(ostr, l);
  ostr << "\n";
  ostr << "bodyPosition:      ";
  printPos(ostr,  fi->bodyBegin());
  l = fi->bodyEnd();
  if ( l.file() && fi->bodyBegin().file() == l.file() )
    ostr << " -> (" << l.line() << "," << l.col() << ")";
  else
    printPos(ostr, l);
  ostr << "\n";
}

void printLo(ostream& ostr, const pdbLoc& l) {
  ostr << "location:          ";
  printPos(ostr, l);
  ostr << "\n";
}

void printSo(ostream& ostr, const pdbFile *f) {
  ostr << "SO#" << f->id() << " " << f->name() << "\n";
  for (pdbFile::incvec::const_iterator it=f->includes().begin();
       it!=f->includes().end(); ++it)
    ostr << "includes:           SO#"
         << (*it)->id() << " " << (*it)->name() << "\n";
  ostr << endl;
}

void printItem(ostream& ostr, const pdbItem *i) {
  ostr << toUpper(i->desc()[0]) << toUpper(i->desc()[1]) << "#" << i->id();
  if ( i->name() == "-" )
    ostr << " <NONAME>\n";
  else
    ostr << " " << i->fullName() << "\n";
  printLo(ostr, i->location());
  if ( const pdbClass* cptr = i->parentClass() ) {
    ostr << "class:              CL#"
         << cptr->id() << " " << cptr->name() << "\n";
    ostr << "access:             " << toName(i->access()) << "\n";
  }
  if ( const pdbNamespace* nptr = i->parentNSpace() ) {
    ostr << "namespace:          NA#"
         << nptr->id() << " " << nptr->name() << "\n";
  }
}

void printTemplItem(ostream& ostr, const pdbTemplateItem *ti, const char *pre) {
  if ( const pdbTemplate* tptr = ti->isTemplate() ) {
    if ( tptr == pdbTemplateItem::TE_UNKNOWN )
      ostr << pre << "isTemplate:         TRUE\n";
    else
      ostr << pre << "instantiatedFrom:   TE#"
           << tptr->id() << "" << tptr->name() << "\n";
    if ( ti->isSpecialized() ) ostr << pre << "isSpecialized:      TRUE\n";
  }
}

void printTy(ostream& ostr, const pdbType *t) {
  if ( t->isClass() ) return;

  pdbType::qualvec q;

  printItem(ostr, t);
  ostr << "kind:               " << toName(t->kind()) << "\n";
  switch ( t->kind() ) {
  case pdbItem::TY_INT:
  case pdbItem::TY_BOOL:
  case pdbItem::TY_WCHAR:
    ostr << "integerKind:        " << toName(t->integerType()) << "\n";
    if ( t->isSigned() ) ostr << "explicitlySigned:   TRUE" << "\n";
    break;
  case pdbItem::TY_ENUM: {
    ostr << "integerKind:        " << toName(t->integerType()) << "\n";
    for (pdbType::enumvec::const_iterator eit=t->enumeration().begin();
         eit!=t->enumeration().end(); ++eit)
      ostr << "enumerationValue:   " << (*eit).id()
           << "=" << (*eit).val() << "\n";
    }
    break;
  case pdbItem::TY_FLOAT:
    ostr << "floatKind:          " << toName(t->floatType()) << "\n";
    break;
  case pdbItem::TY_PTR:
    printType(ostr, "pointedToType:      ", t->pointedToType());
    break;
  case pdbItem::TY_REF:
    printType(ostr, "referencedType:     ", t->referencedType());
    break;
  case pdbItem::TY_ARRAY:
    printType(ostr, "elementType:        ", t->elementType());
    ostr << "numElements:       " << t->numElements() << "\n";
    break;
  case pdbItem::TY_TREF:
    printType(ostr, "baseType:           ", t->baseType());
    q = t->qualifiers();
    if ( q.size() ) {
      ostr << "qualifiers:        ";
      for (pdbType::qualvec::iterator it=q.begin(); it!=q.end(); ++it)
        ostr << " " << toName(*it);
      ostr << "\n";
    }
    break;
  case pdbItem::TY_FUNC: {
    printType(ostr, "returnType:         ", t->returnType());
    for (pdbType::argvec::const_iterator ait=t->argumentTypes().begin();
         ait!=t->argumentTypes().end(); ++ait) {
      printType(ostr, "argumentType:       ", ait->type(),
         ait->hasDefault() ? " hasDefault=TRUE" : " hasDefault=FALSE");
    }
    if ( t->hasEllipsis() ) ostr << "hasEllipsis:        TRUE" << "\n";
    if ( t->isConst() ) ostr << "qualifiers:        const\n";
    for (pdbType::classvec::const_iterator xit=t->exceptionSpec().begin();
         xit!=t->exceptionSpec().end(); ++xit)
      if ( (*xit) == pdbType::NULLEXCEP )
        ostr << "exceptionSpec:      NULL\n";
      else
        ostr << "exceptionSpec:      CL#"
             << (*xit)->id() << " " << (*xit)->name() << "\n";
    }
    break;
  case pdbItem::TY_PTRMEM:  
    ostr << "memberPointerClass: CL#" << t->memberPointerClass()->id()
         << " " << t->memberPointerClass()->name() << "\n";
    printType(ostr, "memberPointerType:  ", t->memberPointerType());
    break;
  default:
    // nothing
    break;
  }
  ostr << endl;
}

void printCf(ostream& ostr, const pdbClassField *c) {
  ostr << "dataMember:         " << c->name() << "\n";
  ostr << "  "; printLo(ostr, c->location());
  if ( const pdbClass* cptr = c->parentClass() ) {
    ostr << "  access:             " << toName(c->access()) << "\n";
  }
  ostr << "  kind:               " << toName(c->kind()) << "\n";
  printType(ostr, "  type:               ", c->type());
  printTemplItem(ostr, c, "  ");
  if ( c->isStaticConst() ) ostr << "  isStaticConst:      TRUE\n";
  if ( c->isBitField() ) ostr << "  isBitfield:         TRUE\n";
  if ( c->isMutable() ) ostr << "  isMutable:          TRUE\n";
}

void printCl(ostream& ostr, const pdbClass *c) {
  printItem(ostr, c);
  ostr << "kind                " << toName(c->kind()) << "\n";
  printTemplItem(ostr, c, "");
  if ( c->isABC() ) ostr << "isAbstractBaseClass:  TRUE\n";
  const pdbClass::basevec b = c->baseClasses();
  for (pdbClass::basevec::const_iterator bt=b.begin(); bt!=b.end(); ++bt) {
     ostr << "baseClass:          "
          << toName((*bt)->virtuality())
          << " " << toName((*bt)->access())
          << " CL#" << (*bt)->base()->id() << " " << (*bt)->base()->name()
          << " ";
     printPos(ostr, **bt);
     ostr << "\n";
  }
  const pdbClass::classvec s = c->derivedClasses();
  for (pdbClass::classvec::const_iterator st=s.begin(); st!=s.end(); ++st)
     ostr << "derivedClass:       CL#"
          << (*st)->id() << " "  << (*st)->name() << "\n";
  const pdbClass::friendclassvec fc = c->friendClasses();
  for (pdbClass::friendclassvec::const_iterator fct=fc.begin();
       fct!=fc.end(); ++fct) {
     ostr << "friendClass:        CL#" << (*fct)->friendClass()->id()
          << " " << (*fct)->friendClass()->name() << " ";
     printPos(ostr, **fct);
     ostr << "\n";
  }
  const pdbClass::friendfuncvec fr = c->friendRoutines();
  for (pdbClass::friendfuncvec::const_iterator frt=fr.begin();
       frt!=fr.end(); ++frt) {
     ostr << "friendRoutine:      RO#" << (*frt)->friendFunc()->id()
          << " " << (*frt)->friendFunc()->name() << " ";
     printPos(ostr, **frt);
     ostr << "\n";
  }
  const pdbClass::methodvec f = c->funcMembers();
  for (pdbClass::methodvec::const_iterator ft=f.begin(); ft!=f.end(); ++ft) {
     ostr << "functionMember:     RO#"
          << (*ft)->func()->id() << " " << (*ft)->func()->name() << " ";
     printPos(ostr, **ft);
     ostr << "\n";
  }
  const pdbClass::fieldvec d = c->dataMembers();
  for (pdbClass::fieldvec::const_iterator dt=d.begin(); dt!=d.end(); ++dt)
     printCf(ostr, *dt);
  printPositions(ostr, c);
  ostr << endl;
}

void printRo(ostream& ostr, const pdbRoutine *r) {
  pdbItem::func_t k;
  pdbItem::virt_t v;
  pdbItem::store_t s;

  printItem(ostr, r);
  ostr << "signature:          TY#" << r->signature()->id() << " "
                          << r->signature()->name() << "\n";
  ostr << "linkage:            " << pdbItem::toName(r->linkage()) << "\n";
  if ( (s = r->storageClass()) != pdbItem::ST_NA )
    ostr << "storageClass:       " << toName(s) << "\n";
  if ( (k = r->specialKind()) != pdbItem::FU_NA )
    ostr << "kind:               " << toName(k) << "\n";
  if ( (v = r->virtuality()) != pdbItem::VI_NO )
    ostr << "virtuality:         " << toName(v) << "\n";
  if ( r->covariantReturnVirtualOverride() )
    ostr << "covariantReturnVirtualOverride:  TRUE\n";
  if ( r->isCompilerGenerated() ) ostr << "isCompilerGenerated:  TRUE\n";
  if ( r->isExplicitCtor() ) ostr << "isExplicitCtor:     TRUE\n";
  if ( r->isInline() ) ostr << "isInline:           TRUE\n";
  if ( r->isStatic() ) ostr << "isStatic:           TRUE\n";
  printTemplItem(ostr, r, "");
  const pdbRoutine::callvec c = r->callees();
  for (pdbRoutine::callvec::const_iterator ct=c.begin(); ct!=c.end(); ++ct) {
    const pdbCallee *func = *ct;
    ostr << "calls:              RO#" << func->call()->id() << " "
         << func->call()->name() << " " << toName(func->virtuality()) << " ";
    printPos(ostr, *func);
    ostr << "\n";
  }
  printPositions(ostr, r);
  ostr << endl;
}

void printTe(ostream& ostr, const pdbTemplate *t) {
  printItem(ostr, t);
  ostr << "kind:               " << toName(t->kind()) << "\n";
  if ( !(t->text() == "") ) ostr << "text:               " << t->text() << "\n";
  printPositions(ostr, t);
  ostr << endl;
}

void printNa(ostream& ostr, const pdbNamespace *n) {
  printItem(ostr, n);
  if ( n->isAlias() )
    ostr << "aliasFor:           NA#"
         << n->isAlias()->id() << " " << n->isAlias()->name() << "\n";
  for (pdbNamespace::memvec::const_iterator it = n->members().begin();
      it!=n->members().end(); ++it) {
    ostr << "member:             "
         << toupper((*it)->desc()[0]) << toupper((*it)->desc()[1]) << "#"
         << (*it)->id() << " " << (*it)->name() << "\n";
  }
  printPositions(ostr, n);
  ostr << endl;
}

void printMa(ostream& ostr, const pdbMacro *m) {
  printItem(ostr, m);
  ostr << "kind:               " << toName(m->kind()) << "\n";
  ostr << "text:               " << m->text() << "\n";
  ostr << endl;
}

void printPDB(PDB& pdb, ostream& ostr) {
  if ( Aflag || Sflag ) {
    PDB::filevec f = pdb.getFileVec();
    for (PDB::filevec::iterator fi = f.begin(); fi != f.end(); ++fi)
      printSo(ostr, *fi);
  }

  if ( Aflag || Yflag ) {
    PDB::typevec t = pdb.getTypeVec();
    for (PDB::typevec::iterator ty = t.begin(); ty != t.end(); ++ty)
      printTy(ostr, *ty);
  }

  if ( Aflag || Cflag ) {
    PDB::classvec c = pdb.getClassVec();
    for (PDB::classvec::iterator cl = c.begin(); cl != c.end(); ++cl)
      printCl(ostr, *cl);
  }

  if ( Aflag || Rflag ) {
    PDB::routinevec r = pdb.getRoutineVec();
    for (PDB::routinevec::iterator ro = r.begin(); ro != r.end(); ++ro)
      printRo(ostr, *ro);
  }

  if ( Aflag || Tflag ) {
    PDB::templatevec u = pdb.getTemplateVec();
    for (PDB::templatevec::iterator te = u.begin(); te != u.end(); ++te)
      printTe(ostr, *te);
  }

  if ( Aflag || Nflag ) {
    PDB::namespacevec n = pdb.getNamespaceVec();
    for (PDB::namespacevec::iterator na = n.begin(); na != n.end(); ++na)
      printNa(ostr, *na);
  }

  if ( Aflag || Mflag ) {
    PDB::macrovec m = pdb.getMacroVec();
    for (PDB::macrovec::iterator ma = m.begin(); ma != m.end(); ++ma)
      printMa(ostr, *ma);
  }
}

int main(int argc, char *argv[]) {
  int ch;
  bool errflag = argc < 2;
  char *outfile = 0;

  while ( (ch = getopt(argc, argv, "ACMNRSTYo:")) != -1 ) {
    switch (ch) {
    case 'A': Aflag = true;     break;
    case 'C': Cflag = true;     break;
    case 'M': Mflag = true;     break;
    case 'N': Nflag = true;     break;
    case 'R': Rflag = true;     break;
    case 'S': Sflag = true;     break;
    case 'T': Tflag = true;     break;
    case 'Y': Yflag = true;     break;
    case 'o': outfile = optarg; break;
    case '?': errflag = true;   break;
    }
  }
  if ( errflag ) {
    cerr << "usage: " << argv[0] << " [-o outfile] [-ACMNRSTY] pdbfile" << endl;
    return 1;
  }

  PDB pdb(argv[optind]);
 
  ostream* ostr;
  ofstream of;
  if ( outfile ) {
    of.open(outfile);
    ostr = &of;
  } else 
    ostr = &cout;

  if ( pdb ) {
    if ( Aflag || Cflag || Mflag || Nflag ||
         Rflag || Sflag || Tflag || Yflag )
      printPDB(pdb, *ostr);
    else
      pdb.write(*ostr);
    return 0;
  }
  return 1;
}
